from django.apps import AppConfig
from django.conf import settings
from corsheaders.defaults import default_headers
from ethosinnovations.helpers.utils import generate_origins

import environ


class EthosInnovationsConfig(AppConfig):
    name = "ethosinnovations"
    verbose_name = "Ethos Innovations Utilities"

    def ready(self):
        """Automatically load AWS S3 settings into Django settings using django-environ."""

        # Initialize environ
        env = environ.Env()
        environ.Env.read_env()  # Load from .env file if available

        AWS_ACCESS_KEY_ID = env("AWS_ACCESS_KEY_ID", default=None)
        AWS_SECRET_ACCESS_KEY = env("AWS_SECRET_ACCESS_KEY", default=None)
        AWS_SESSION_TOKEN = env("AWS_SESSION_TOKEN", default=None)

        # AWS S3 Configuration
        # AWS S3 Additional Configuration
        AWS_S3_REGION_NAME = "ap-southeast-1"

        # Static Files Configuration
        AWS_S3_STATIC_BUCKET = env("AWS_S3_STATIC_BUCKET")
        AWS_S3_STATIC_CUSTOM_DOMAIN = env(
            "AWS_S3_STATIC_CUSTOM_DOMAIN",
            default=f"{AWS_S3_STATIC_BUCKET}.s3.amazonaws.com",
        )
        STATIC_LOCATION = "static"
        STATIC_URL = f"https://{AWS_S3_STATIC_CUSTOM_DOMAIN}/{STATIC_LOCATION}/"

        # Media Files Configuration
        AWS_S3_MEDIA_BUCKET = env("AWS_S3_MEDIA_BUCKET")
        AWS_S3_MEDIA_CUSTOM_DOMAIN = env(
            "AWS_S3_MEDIA_CUSTOM_DOMAIN",
            default=f"{AWS_S3_MEDIA_BUCKET}.s3.{AWS_S3_REGION_NAME}.amazonaws.com",
        )
        MEDIA_URL = f"https://{AWS_S3_MEDIA_CUSTOM_DOMAIN}/"

        STORAGES = {
            "default": {
                "BACKEND": "storages.backends.s3boto3.S3Boto3Storage",
                "OPTIONS": {
                    "bucket_name": AWS_S3_MEDIA_BUCKET,
                    "region_name": AWS_S3_REGION_NAME,
                    "querystring_auth": False,
                    "custom_domain": AWS_S3_MEDIA_CUSTOM_DOMAIN,
                },
            },
            "staticfiles": {
                "BACKEND": "storages.backends.s3boto3.S3Boto3Storage",
                "OPTIONS": {
                    "bucket_name": AWS_S3_STATIC_BUCKET,
                    "region_name": AWS_S3_REGION_NAME,
                    "querystring_auth": False,
                    "custom_domain": AWS_S3_STATIC_CUSTOM_DOMAIN,
                },
            },
        }

        DEBUG = env("DEBUG", default=False)
        cors_origins = generate_origins(
            domains=settings.DOMAINS,
            protocols=["https", "http"],
            include_localhost=DEBUG,
        )

        CORS_ALLOWED_ORIGINS = cors_origins
        CSRF_TRUSTED_ORIGINS = cors_origins

        ALLOWED_HOSTS = generate_origins(
            domains=settings.DOMAINS,
            protocols=None,
            include_localhost=DEBUG,
        )
        CORS_ALLOW_HEADERS = list(default_headers)

        MIDDLEWARE = [
            "corsheaders.middleware.CorsMiddleware",
            "ethosinnovations.middlewares.HealthCheckMiddleware",
            *settings.MIDDLEWARE,
        ]

        # Default settings
        default_settings = {
            "AWS_ACCESS_KEY_ID": AWS_ACCESS_KEY_ID,
            "AWS_SECRET_ACCESS_KEY": AWS_SECRET_ACCESS_KEY,
            "AWS_SESSION_TOKEN": AWS_SESSION_TOKEN,
            "AWS_S3_REGION_NAME": AWS_S3_REGION_NAME,
            "AWS_S3_STATIC_BUCKET": AWS_S3_STATIC_BUCKET,
            "AWS_S3_STATIC_CUSTOM_DOMAIN": AWS_S3_STATIC_CUSTOM_DOMAIN,
            "STATIC_LOCATION": STATIC_LOCATION,
            "STATIC_URL": STATIC_URL,
            "AWS_S3_MEDIA_BUCKET": AWS_S3_MEDIA_BUCKET,
            "AWS_S3_MEDIA_CUSTOM_DOMAIN": AWS_S3_MEDIA_CUSTOM_DOMAIN,
            "MEDIA_URL": MEDIA_URL,
            "STORAGES": STORAGES,
            "DEBUG": DEBUG,
            "CORS_ALLOWED_ORIGINS": CORS_ALLOWED_ORIGINS,
            "CSRF_TRUSTED_ORIGINS": CSRF_TRUSTED_ORIGINS,
            "ALLOWED_HOSTS": ALLOWED_HOSTS,
            "CORS_ALLOW_HEADERS": CORS_ALLOW_HEADERS,
            "MIDDLEWARE": MIDDLEWARE,
        }

        # Inject settings into Django settings if they are missing
        for setting, default_value in default_settings.items():
            if not hasattr(settings, setting):
                setattr(settings, setting, default_value)
