import boto3
from botocore.exceptions import ClientError
from django.conf import settings
from urllib.parse import urlparse

from .exceptions import S3UploadError


class S3Client:
    def __init__(self):
        AWS_ACCESS_KEY_ID = settings.AWS_ACCESS_KEY_ID
        AWS_SECRET_ACCESS_KEY = settings.AWS_SECRET_ACCESS_KEY
        AWS_SESSION_TOKEN = (
            settings.AWS_SESSION_TOKEN
            if hasattr(settings, "AWS_SESSION_TOKEN")
            else None
        )

        if AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY:
            # Use provided access keys
            session = boto3.Session(
                aws_access_key_id=AWS_ACCESS_KEY_ID,
                aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
                aws_session_token=AWS_SESSION_TOKEN,
            )
            self.s3_client = session.client("s3")
        else:
            # Use IAM role attached to the ECS task
            self.s3_client = boto3.client("s3")

        self.AWS_BUCKET_NAME = settings.AWS_S3_MEDIA_BUCKET

    def upload_to_s3(self, file, object_name: str, is_fileobj: bool = False) -> str:
        """
        Upload file to AWS S3.
        """
        try:
            if is_fileobj:
                self.s3_client.upload_fileobj(
                    Fileobj=file,
                    Bucket=self.AWS_BUCKET_NAME,
                    Key=object_name,
                )
            else:
                self.s3_client.upload_file(
                    Filename=file,
                    Bucket=self.AWS_BUCKET_NAME,
                    Key=object_name,
                )
        except ClientError as e:
            raise S3UploadError(f"Error uploading file to S3: {repr(e)}")
        except Exception as e:
            raise Exception(f"Error uploading file to S3: {repr(e)}")

        return f'{settings.MEDIA_URL}{object_name.lstrip("/")}'

    def create_presigned_post(self, object_name: str, expiration: int = 300) -> dict:
        """
        Generate a presigned URL S3 POST request to upload a file

        :param expiration: Time in seconds
        :return:
            url: URL to post to
            fields: Dictionary of form fields and values to submit with the POST
        """
        try:
            presigned_url = self.s3_client.generate_presigned_post(
                Bucket=self.AWS_BUCKET_NAME,
                Key=object_name,
                ExpiresIn=expiration,
            )
        except ClientError as e:
            print(f"Error generating presigned URL: {repr(e)}")

        return {
            **presigned_url,
            "cloudfront_url": f"{settings.MEDIA_URL}{object_name.lstrip('/')}",
        }

    def delete_from_s3(self, file_url: str) -> bool:
        """
        Delete a file from AWS S3 using the file URL.

        :param file_url: Full URL of the file in the S3 bucket
        :return: True if file was successfully deleted, otherwise False
        """
        try:
            # Parse the file URL to get the object key
            parsed_url = urlparse(file_url)
            object_key = parsed_url.path.lstrip("/")  # Remove leading slash if exists

            # Delete the object from the S3 bucket
            self.s3_client.delete_object(Bucket=self.AWS_BUCKET_NAME, Key=object_key)
            return True

        except ClientError as exc:
            print(f"Error deleting {file_url}: {repr(exc)}")
            return False

    def move_s3_file(self, source_file_url: str, destination_object_name: str) -> str:
        """
        Move a file within the S3 bucket by copying it to a new location and deleting the original.

        :param source_file_url: The full URL of the source file in S3.
        :param destination_object_name: The destination key (path) within the S3 bucket.
        :return: The URL of the file at the new location if successful, or None if failed.
        """
        try:
            # Parse the source URL to extract the object key (path in S3)
            parsed_url = urlparse(source_file_url)
            bucket_name = self.AWS_BUCKET_NAME
            source_object_key = parsed_url.path.lstrip(
                "/"
            )  # Remove leading slash if exists

            # Define the copy source (bucket and key)
            copy_source = {"Bucket": bucket_name, "Key": source_object_key}

            # Perform the copy operation
            self.s3_client.copy_object(
                CopySource=copy_source, Bucket=bucket_name, Key=destination_object_name
            )

            # After successfully copying, delete the original file using the existing delete_from_s3 function
            if not self.delete_from_s3(source_file_url):
                print(
                    f"Warning: Failed to delete the original file {source_file_url} after copying."
                )

            # Return the final URL of the moved file
            final_file_url = (
                f"https://{bucket_name}.s3.amazonaws.com/{destination_object_name}"
            )
            return final_file_url

        except ClientError as exc:
            print(
                f"Error moving file from {source_file_url} to {destination_object_name}: {repr(exc)}"
            )
            return None

    def generate_presigned_url(self, object_name: str, expiration: int = 3600) -> str:
        """
        Generate a presigned URL to download an S3 file.

        :param object_name: The S3 object name (key).
        :param expiration: The time in seconds for the presigned URL to remain valid.
        :return: The presigned URL if successful, else None.
        """
        try:
            # Generate the presigned URL for 'get_object'
            presigned_url = self.s3_client.generate_presigned_url(
                "get_object",
                Params={"Bucket": self.AWS_BUCKET_NAME, "Key": object_name},
                ExpiresIn=expiration,
            )
            return presigned_url
        except ClientError as exc:
            print(f"Error generating presigned URL for {object_name}: {repr(exc)}")
            return None

    def get_s3_key_from_url(self, s3_url):
        """
        Extract the S3 key (object path) from the full S3 URL.
        """
        parsed_url = urlparse(s3_url)
        # Remove leading '/' to get the object key
        return parsed_url.path.lstrip("/")
