import csv

from .echo import Echo


def csv_generator(data):
    """
    A generator that yields CSV formatted strings.

    :param data: The data rows (including headers) to format as CSV.
    """
    writer = csv.writer(Echo())
    for row in data:
        # csv.writer expects a list, but writerow returns the row string,
        # so we directly pass the row to Echo's write method.
        yield writer.writerow(row)


def stream_csv_data(queryset, csv_fields, batch_size=10000):
    """
    A generator that yields CSV data rows from a queryset in bulk batches.

    :param queryset: The queryset to process for CSV export.
    :param csv_fields: The fields to include in the CSV export.
    :param batch_size: Number of rows to fetch in each batch.
    """
    # Yield the header row
    headers = [field["header_name"] for field in csv_fields]
    yield headers

    # Use iterator() for memory efficiency
    for obj in queryset.iterator(chunk_size=batch_size):
        data_row = []
        for field in csv_fields:
            if "getter" in field:
                # Use the getter function if provided
                value = field["getter"](obj)
            elif "field_name" in field:
                # Use standard field lookup if field_name is provided
                value = _get_field_value(obj, field["field_name"])
            else:
                value = ""
            data_row.append(value if value is not None else "")
        yield data_row


def _get_field_value(obj, field_name):
    """Helper function to get field value using Django's double-underscore notation."""
    if isinstance(field_name, tuple):
        # Handle tuple fields (concatenation)
        values = []
        for part in field_name:
            value = _get_single_field_value(obj, part)
            if value:
                values.append(str(value))
        return " ".join(values).strip()
    return _get_single_field_value(obj, field_name)


def _get_single_field_value(obj, field_name):
    """Get a single field value using Django's double-underscore notation."""
    try:
        for part in field_name.split("__"):
            obj = getattr(obj, part)
        return obj
    except (AttributeError, TypeError):
        return None
