def attach_domain_urls(domain, protocols=None, subdomains=None):
    """
    Generate a list of URLs for allowed origins with the given subdomains.
    Supports multiple protocols or omits them if None.
    Defaults to 'www', 'dev', and '' subdomains.
    """
    # Default subdomains if not provided
    subdomains = subdomains or ["www", "dev", "api", "api.dev", ""]

    # Generate URLs without protocol
    if protocols is None:
        return [f"{sub + '.' if sub else ''}{domain}" for sub in subdomains]

    # Generate URLs with specified protocols
    return [
        f"{protocol}://{sub + '.' if sub else ''}{domain}"
        for sub in subdomains
        for protocol in protocols
    ]


def generate_origins(domains, protocols=None, include_localhost=True):
    """
    Generate a comprehensive list of origins from a list of domains.
    - Supports multiple protocols (https, http) or omits them if None.
    - Optionally includes localhost with correct formatting.
    """
    origins = []

    for domain in domains:
        origins.extend(attach_domain_urls(domain, protocols))

    if include_localhost:
        localhost_hosts = [
            "localhost",
            "127.0.0.1",
            "0.0.0.0",
            "::1",
            "0:0:0:0:0:0:0:1",
        ]

        # Add hosts with and without port 3000
        hosts_with_ports = []
        for host in localhost_hosts:
            hosts_with_ports.append(host)
            hosts_with_ports.append(f"{host}:3000")

        if protocols is None:
            origins.extend(hosts_with_ports)
        else:
            for protocol in protocols:
                for host in hosts_with_ports:
                    origins.append(f"{protocol}://{host}")

    return origins
